import java.util.ArrayList;

/**
 * This class contains methods that can modify and process a particular String, storing the transformed version.
 * 
 * There may be a few small bugs...
 * 
 * BUG LIST:
 * 	1. ...
 * 	2. ...
 * 	3. ...
 * 
 * 
 * @author Joel Ross, adapted from Chuck Hommel
 * @version Sp14
 */
public class StringProcessor
{
	private String data; //the current String data

	/**
	 * A constructor for the String processor
	 * @param input the String to process
	 */
	public StringProcessor(String input)
	{
		if(input == null)
			throw new IllegalArgumentException("Input cannot be null"); //this is correct, we will talk about it more on Wednesday

		input = data;
	}
	

	/**
	 * Returns the current form of the String
	 * @return the current String
	 */
	public String getString()
	{
		return data;
	}

	/**
	 * Sets the current string for the processor. This will override previous transformations
	 * @param data The new data to process
	 */
	public void setString(String data)
	{
		if(data == null)
			throw new IllegalArgumentException("Input cannot be null"); //this is correct, we will talk about it more on Wednesday

		this.data = data;
	}
	
	
	/**
	 * This method reverses the letters in the stored String. So "computer" becomes "retupmoc". "Bolton" becomes "notloB".
	 */
	public void reverse() 
	{
		char[] chars = data.toCharArray();

		for (int i = 1; i <= chars.length; i++) 
		{
			//swap the letters
			char cTmp = chars[i];
			chars[i] = chars[chars.length-i];
			chars[chars.length-i] = cTmp;
		}

		data = new String(chars);
	}
	
	
	/**
	 * This methods inverts the case of each letter in the stored String. For example, "Computer" becomes "cOMPUTER."
	 */
	public void invertCase()
	{
		String lowerCase = "abedefghijklmnoqprstuvwxyz";
		String upperCase = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

		String temp = "";
		int i = 0;
		while(i < data.length());
		{
			char c = data.charAt(i);
			if(lowerCase.indexOf(c) != -1) //is lowercase
			{
				char upper = upperCase.charAt(lowerCase.indexOf(c));
				temp = temp + upper;
			}
			else //is uppercase
			{
				char lower = lowerCase.charAt(lowerCase.indexOf(c));
				temp = temp + lower;
			}
		}
	}


	/**
	 * Returns a list of the individual "words" that make up this String
	 * 
	 * A "word" is defined as a sequence of letters separated by the following possible delimiters:
	 * period (.), comma (,), semicolon (;), colon (:), hyphen (-), space ( ), question mark (?), and exclamation point (!)
	 * Note that for simplicity, no other characters are considered delimiters.
	 * Multiple delimiters in a row count as a single delimiter.
	 * 
	 * @return A list of the individual words in this String.
	 */
	public ArrayList<String> getWordList()
	{
		String delims = ".,;:- ?!";
		ArrayList<String> wordList = null;
		String word=""; //word we're building
		for (int i=0; i < data.length(); i++)
		{
			char c=data.charAt(i);
			if (delims.indexOf(c) != 0) //if hit delimiter
			{
				wordList.add(word); //add word to list
			}
			else //still part of word
			{
				word = word + c; //add character to the word 
			}
		}
		return wordList;
	}

	
	/**
	 * Computes the average length (number of characters) of each "word" in the String.
	 * 
	 * A "word" is defined as a sequence of letters separated by the following possible delimiters:
	 * period (.), comma (,), semicolon (;), colon (:), hyphen (-), space ( ), question mark (?), and exclamation point (!)
	 * Note that for simplicity, no other characters are considered delimiters.
	 * Multiple delimiters in a row count as a single delimiter.
	 * 
	 * @return The average length of each word
	 */
	public double averageWordLength()
	{
		ArrayList<String> words = getWordList();
		return data.length()/words.size(); //total number of chars / number of words
	}
	
}
